
/* Copyright (c) 1996 Dennis P. Glatting
        dennis.glatting@plaintalk.bellevue.wa.us
    All rights reserved.

    The author disclaims all responsibility or liability with 
    respect to it's usage or its effect upon hardware or 
    computer systems, and maintains copyright as set out in
    the "Artistic" license document which accompanies 
    distributions of PERL 5. 

    This program is used in conjunction with the program that
    generates many, many Crack rules. Its input is a file
    generated by the rule generation program. Its output, to
    stdout, is the contents of the rule file with the rules
    sorted by ascending length. 

    The purpose of the sort is to insure the rules with the
    highest probability of not being empty when applied to a
    dictionary, are among the first processed by Crack. 
 
 */


#include    <assert.h>
#include    <errno.h>
#include    <memory.h>
#include    <stdio.h>
#include    <stdlib.h>
#include    <sys/types.h>


/* This structure holds the length of a
    rule and its position in the rule file. */ 

typedef struct _elem {

    size_t      len;
    long int    pos;

} elem, * elem_t;


/* This definition is the name of 
    the rule file that is sorted. */

#define RULE_FILE   \
    "/ExternalDisk/security/src/crack.4.1f/Scripts/dicts.rules.only.subs"


/* This function is called by qsort() to 
    sort the array of "elem" structures. */

int
compar( const void * e1, const void * e2 ) {


    assert( e1 && e2 );
    

    return ((const elem_t)e1)->len - ((const elem_t)e2)->len;
}


int
main ( int argc, const char ** argv, const char ** envp ) {

    int     rval = 0;
    FILE *  fp = NULL;
    elem    (*dp)[] = NULL;
    
    
    if(( fp = fopen( RULE_FILE, "r" ))) {
    
        u_int   num_rules = 0, indx = 0;
        char    buf[ 4096 ];
        
            /* Clear the buffer. The last byte of the buffer 
                isn't used and thus is always 0. If one of 
                the rules is longer than the buffer size then 
                the 0 will be missing after the read and an 
                assertion executed. */
            
        memset( buf, sizeof( buf ), 0 );
        
            /* Read the rule file and count the number of rules. */
            
        while( fgets( buf, sizeof( buf ) - 1, fp )) {
        
            assert( strlen( buf ) < sizeof( buf ));
            
            ++num_rules;
        
        }
        
        if( num_rules == 0 ) {
        
            fprintf( stderr, "Rule file is empty.\n" );
            
            rval = -1;
            goto clean_exit;
    
        }

            /* Need something to hold the rule data. */
            
        dp = calloc( num_rules, sizeof( elem ));
        if( ! dp ) {
        
            fprintf( stderr, "Not enough memory to hold rule data.\n" );

            rval = errno;
            goto clean_exit;
        }
        
            /* Rewind the file, read each rule anew, and 
                store the rule's length and position in 
                the file into the array. */
    
        rewind( fp );
            
        for( indx = 0; indx < num_rules; ++indx ) {

            char *  fgets_err = NULL;

            (*dp)[ indx ].pos = ftell( fp );
            assert( (*dp)[ indx ].pos != -1 );

            fgets_err = fgets( buf, sizeof( buf ), fp );
            assert( fgets_err );
            
            (*dp)[ indx ].len = strlen( buf );
                    
        }
        
            /* Sort the rules based on their length. */
            
        qsort( dp, num_rules, sizeof( elem ), compar );
            
            /* Rewind the file and iterate through the sorted 
                array. For each entry in the array seek to 
                the rules position in the file, read it, and 
                print it. */
            
        rewind( fp );
        
        for( indx = 0; indx < num_rules; ++indx ) {
            
            int     fseek_err;
            char *  fgets_err = NULL;

            fseek_err = fseek( fp, (*dp)[ indx ].pos, SEEK_SET );
            assert( fseek_err != -1 );
                
            fgets_err = fgets( buf, sizeof( buf ), fp );
            assert( fgets_err );
                
            fputs( buf, stdout );
    
        }       
        
    } else {
    
        fprintf( stderr, "Unable to open input rile file.\n" );

        rval = errno;
        goto clean_exit;
    
    }


clean_exit:

    if( fp )
        fclose( fp ),
            fp = NULL;
    
    if( dp ) 
        free( dp ),
            dp = NULL;

    return rval;
}



